;
;	>>> ISO-System by JAC! <<<
;
;	Created for Sillyventure 2011 in Gdansk.
;
;	This file contains the system code which must be there on all banks.
;
;	@com.wudsn.ide.asm.mainsourcefile=ISO.asm

	.local system		;Common code for all banks

;===============================================================

return	= wait.return		;Common return statement in all banks

;===============================================================

	.proc wait		;Delay a defined number of cycles
wait24	nop
wait22	nop
wait20	nop
wait18	nop
wait16	nop
wait14	nop
wait12
return	rts
	.endp			;End of wait

;===============================================================

;	Set object horizontal position
;	IN:  <A>=0..159
;	IN:  <X>=offset for "sta resp0,x", 0=P0, ...
;	OUT: <X> increased by 1, <A>=fine positioning value, C=0, Z=0


; 	lda #xxx		;Position between 0-159
;	ldx #object.p0		;Offset for resp0,x
;	jsr object.set_set_hpos	;Set position
;	...			;Set more position
;	jsr object.move_hpos	;Perform WSYNC/HMOVE/HMCLR

	.proc object		;Move objects
	p0 = 0	;Player 0
	p1 = 1	;Player 1
	m0 = 2	;Missle 0
	m1 = 3	;Missle 1
	bl = 4	;Ball

	.proc set_hpos		;Set object horizontal position
	.if .def set_hpos_sec_first
	.print "Using hpos with SEC first"
	sec			;2
	sta wsync		;3
	.else
	.print "Using hpos with WSYNC first"
	sta wsync		;3
	sec			;2
	.endif
without_sync			;Start without sync, carry and <A> must be adjusted accordingly
loop	sbc #$0f		;2 => 5*3 pixel = 15 pixel per interation 
	bcs loop			;2 => 5 cycles per iteration, branch must not cross page-boundary
	eor #$07		;2
	asl			;2
	asl			;2
	asl			;2
	asl			;2
	sta resp0,x		;4
	sta hmp0,x		;4 => Fine positioning value
	inx			;2
	rts			;6
	.endp			;End of set_hpos

	assert_same_page set_hpos

	.proc move_hpos		;Store wsync, hmove and hmclr
	sta wsync
	sta hmove
	jsr wait.wait20
	sta hmclr
	rts
	.endp			;End of move_hpos
	
	.endp			;End of object

;===============================================================

	.proc screen_off	;Turn the screen off
with_sync
	sta wsync
without_sync
	lda #0
	sta colubk
	sta pf0
	sta pf1
	sta pf2
	sta grp0
	sta grp1
	sta enam0
	sta enam1
	sta enabl
	sta colup0
	sta colup1
	sta colupf
	rts
	.endp			;End of screen_off

;===============================================================

	.proc call_vblank	;Indirect call with automatic bank switching, to be called from bank 1
	bit vblank_ptr+1	;Bank 1 is in $7000-7fff, bank2 in $f000-ffff
	bpl is_bank1
	bit mmu.bank2
is_bank1
	jsr call
	bit mmu.bank1
	jmp  bank1.cart.main.vblank_area.return

call	jmp (vblank_ptr)
	.endp

;===============================================================

	.proc call_playfield	;Indirect call with automatic bank switching, to be called from bank 1
	bit playfield_ptr+1	;Bank 1 is in $7000-7fff, bank2 in $f000-ffff
	bpl is_bank1
	bit mmu.bank2
is_bank1
	jsr call
	bit mmu.bank1
	jmp bank1.cart.main.playfield.return

call	jmp (playfield_ptr)
	.endp

;===============================================================

	.proc call_animate	;Indirect call with automatic bank switching, to be called from bank 1
	bit animate_ptr+1	;Bank 1 is in $7000-7fff, bank2 in $f000-ffff
	bpl is_bank1
	bit mmu.bank2
is_bank1
	jsr call
	bit mmu.bank1
	rts

call	jmp (animate_ptr)
	.endp

;===============================================================

	.endl			;End of system

;===============================================================

	.proc start		;Start of the cartridge
	bit mmu.bank1		;Activate bank1
	.endp
				;Cart start must be next in bank1

